@@ -20,14 +20,12 @@
 #include <string.h>
 #include <ctype.h>
 #include "srconfig.h"
+#include "mchar.h"
 #include "debug.h"
 #include "srtypes.h"
-#include "util.h"
 #include "regex.h"
 
-/*****************************************************************************
- * Public functions
- *****************************************************************************/
+/* GCS FIX: figure out function prototypes for regwcomp, regwexec */
 
 /*****************************************************************************
  * Private global variables
@@ -53,46 +51,91 @@ struct parse_rule {
     int album_idx;
     int trackno_idx;
     regex_t* reg;
-    char* match;
-    char* subst;
+    mchar* match;
+    mchar* subst;
 };
 typedef struct parse_rule Parse_Rule;
 
 static Parse_Rule m_default_rule_list[] = {
     { PARSERULE_CMD_MATCH, 
-	PARSERULE_SKIP,
-	0, 0, 0, 0,
-	0, 
-	"^A suivre:",
-	""
+      PARSERULE_SKIP,
+      0, 0, 0, 0,
+      0, 
+#if defined HAVE_WCHAR_SUPPORT
+      /* GCS FIX: L string literals are not good */
+      L"^A suivre:",
+      L""
+#else
+      "^A suivre:",
+      ""
+#endif
     },
     { PARSERULE_CMD_SUBST, 
-	PARSERULE_ICASE,
-	0, 0, 0, 0,
-	0, 
-	"[[:space:]]*-?[[:space:]]*mp3pro$",
-	""
+      PARSERULE_ICASE,
+      0, 0, 0, 0,
+      0, 
+#if defined HAVE_WCHAR_SUPPORT
+      L"[[:space:]]*-?[[:space:]]*mp3pro$",
+      L""
+#else
+      "[[:space:]]*-?[[:space:]]*mp3pro$",
+      ""
+#endif
     },
     { PARSERULE_CMD_MATCH, 
-	PARSERULE_ICASE,
-	1, 2, 0, 0,
-	0, 
-	"^[[:space:]]*([^-]*[^-[:space:]])[[:space:]]*-[[:space:]]*(.*)[[:space:]]*$",
-	""
+      PARSERULE_ICASE,
+      1, 2, 0, 0,
+      0, 
+#if defined HAVE_WCHAR_SUPPORT
+      L"^[[:space:]]*([^-]*[^-[:space:]])[[:space:]]*-[[:space:]]*(.*)[[:space:]]*$",
+      L""
+#else
+      "^[[:space:]]*([^-]*[^-[:space:]])[[:space:]]*-[[:space:]]*(.*)[[:space:]]*$",
+      ""
+#endif
     },
     { 0x00, 
-	0x00,
-	0, 0, 0, 0,
-	0, 
-	"",
-	""
+      0x00,
+      0, 0, 0, 0,
+      0, 
+#if defined HAVE_WCHAR_SUPPORT
+      L"",
+      L""
+#else
+      "",
+      ""
+#endif
     }
 };
 static Parse_Rule* m_global_rule_list = m_default_rule_list;
 
+/*****************************************************************************
+ * Public functions
+ *****************************************************************************/
+static int
+sr_regcomp (Parse_Rule* pr, mchar* rule_string, int cflags)
+{
+#if defined HAVE_WCHAR_SUPPORT
+    return regwcomp(pr->reg, rule_string, cflags);
+#else
+    return regcomp(pr->reg, rule_string, cflags);
+#endif
+}
+
+static int
+mregexec (const regex_t* preg, const mchar* string, size_t nmatch,
+	  regmatch_t pmatch[], int eflags)
+{
+#if defined HAVE_WCHAR_SUPPORT
+    return regwexec (preg, string, nmatch, pmatch, eflags);
+#else
+    return regexec (preg, string, nmatch, pmatch, eflags);
+#endif
+}
+
 /* Returns 1 if successful, 0 if failure */
 static int
-compile_rule (Parse_Rule* pr, char* rule_string)
+compile_rule (Parse_Rule* pr, mchar* rule_string)
 {
     int rc;
     int cflags;
@@ -104,10 +147,8 @@ compile_rule (Parse_Rule* pr, char* rule_string)
     if (pr->flags & PARSERULE_ICASE) {
 	cflags |= REG_ICASE;
     }
-    rc = regcomp(pr->reg, rule_string, cflags);
+    rc = sr_regcomp(pr, rule_string, cflags);
     if (rc != 0) {
-	printf ("Warning: malformed regular expression:\n%s\n",
-	    rule_string);
 	free(pr->reg);
 	return 0;
     }
@@ -129,11 +170,12 @@ use_default_rules (void)
 }
 
 static void
-copy_rule_result (char* dest, char* query_string, regmatch_t* pmatch, int idx)
+copy_rule_result (mchar* dest, mchar* query_string, 
+		  regmatch_t* pmatch, int idx)
 {
     if (idx > 0 && idx <= MAX_SUBMATCHES) {
-	sr_strncpy(dest, query_string + pmatch[idx].rm_so,
-	    pmatch[idx].rm_eo - pmatch[idx].rm_so + 1);
+	mstrncpy (dest, query_string + pmatch[idx].rm_so,
+		  pmatch[idx].rm_eo - pmatch[idx].rm_so + 1);
     }
 }
 
@@ -253,13 +295,15 @@ init_metadata_parser (char* rules_file)
 	char rule_buf[MAX_RULE_SIZE];
 	char match_buf[MAX_RULE_SIZE];
 	char subst_buf[MAX_RULE_SIZE];
+	mchar w_match_buf[MAX_RULE_SIZE];
+	mchar w_subst_buf[MAX_RULE_SIZE];
 	char* rbp;
 	int got_command;
 	int rc;
 	
 	/* Allocate memory for rule */
 	m_global_rule_list = realloc (m_global_rule_list,
-	    (ri+1) * sizeof(Parse_Rule));
+				      (ri+1) * sizeof(Parse_Rule));
 	memset (&m_global_rule_list[ri], 0, sizeof(Parse_Rule));
 
 	/* Get next line from file */
@@ -287,7 +331,7 @@ init_metadata_parser (char* rules_file)
 	default:
 	    got_command = 0;
 	    printf ("Warning: malformed command in rules file:\n%s\n",
-		rule_buf);
+		    rule_buf);
 	    break;
 	}
 	if (!got_command) continue;
@@ -295,24 +339,26 @@ init_metadata_parser (char* rules_file)
 	/* Skip past fwd slash */
 	if (*rbp++ != '/') {
 	    printf ("Warning: malformed command in rules file:\n%s\n",
-		rule_buf);
+		    rule_buf);
 	    continue;
 	}
 
 	/* Parse match string */
 	rbp = parse_escaped_string (match_buf, rbp);
-	    if (!rbp) {
-		printf ("Warning: malformed command in rules file:\n%s\n",
+	debug_printf ("match_buf=%s\n", match_buf);
+	if (!rbp) {
+	    printf ("Warning: malformed command in rules file:\n%s\n",
 		    rule_buf);
-		continue;
-	    }
+	    continue;
+	}
 
 	/* Parse subst string */
 	if (m_global_rule_list[ri].cmd == PARSERULE_CMD_SUBST) {
 	    rbp = parse_escaped_string (subst_buf, rbp);
+	    debug_printf ("subst_buf=%s\n", subst_buf);
 	    if (!rbp) {
 		printf ("Warning: malformed command in rules file:\n%s\n",
-		    rule_buf);
+			rule_buf);
 		continue;
 	    }
 	}
@@ -321,39 +367,71 @@ init_metadata_parser (char* rules_file)
 	rc = parse_flags (&m_global_rule_list[ri], rbp);
 	if (!rc) {
 	    printf ("Warning: malformed command in rules file:\n%s\n",
-		rule_buf);
+		    rule_buf);
 	    continue;
 	}
 
 	/* Compile the rule */
-	if (!compile_rule(&m_global_rule_list[ri], match_buf)) continue;
-
-	/* Copy rule strings */
-	m_global_rule_list[ri].match = strdup(match_buf);
-	if (m_global_rule_list[ri].cmd == PARSERULE_CMD_SUBST) {
-	    m_global_rule_list[ri].subst = strdup(subst_buf);
+	debug_printf ("Compiling the rule\n");
+	mstring_from_string (w_match_buf, MAX_RULE_SIZE, match_buf, 
+			     CODESET_UTF8);
+	if (!compile_rule(&m_global_rule_list[ri], w_match_buf)) {
+	    printf ("Warning: malformed regular expression:\n%s\n", 
+		    match_buf);
+	    continue;
 	}
 
+	/* Copy rule strings */
+	debug_printf ("Copying rule string (1)\n");
+	debug_mprintf (m_("String is ") m_S m_("\n"), w_match_buf);
+	m_global_rule_list[ri].match = mstrdup(w_match_buf);
+	debug_printf ("Copying rule string (2)\n");
+	if (m_global_rule_list[ri].cmd == PARSERULE_CMD_SUBST) {
+	    debug_printf ("Copying rule string (3)\n");
+	    mstring_from_string (w_subst_buf, MAX_RULE_SIZE, subst_buf, 
+				 CODESET_UTF8);
+	    debug_printf ("Copying rule string (4)\n");
+	    m_global_rule_list[ri].subst = mstrdup(w_subst_buf);
+	    debug_printf ("Copying rule string (5)\n");
+	}
+
+	debug_printf ("End of loop\n");
 	ri++;
     }
     /* Add null rule to the end */
     m_global_rule_list = realloc (m_global_rule_list,
-	(ri+1) * sizeof(Parse_Rule));
+				  (ri+1) * sizeof(Parse_Rule));
     memset (&m_global_rule_list[ri], 0, sizeof(Parse_Rule));
 }
 
 void
 compose_metadata (TRACK_INFO* ti)
 {
+    int num_bytes;
+    unsigned char num_16_bytes;
+    mchar w_composed_metadata[MAX_METADATA_LEN+1];
+
     if (ti->have_track_info) {
 	if (ti->artist[0]) {
-	    sprintf (ti->composed_metadata, "StreamTitle='%s - %s';",
-		     ti->artist, ti->title);
+	    msnprintf (w_composed_metadata, MAX_METADATA_LEN,
+		       m_("StreamTitle='") m_S m_(" - ") m_S m_("';"),
+		       ti->artist, ti->title);
 	} else {
-	    sprintf (ti->composed_metadata, "StreamTitle='%s';",
-		     ti->title);
+	    msnprintf (w_composed_metadata, MAX_METADATA_LEN,
+		       m_("StreamTitle='") m_S m_("';"),
+		       ti->title);
 	}
+    } else {
+	debug_printf ("No track info when composing relay metadata\n");
     }
+    debug_printf ("Converting relay string to char\n");
+    num_bytes = string_from_mstring (&ti->composed_metadata[1], 
+				     MAX_METADATA_LEN, 
+				     w_composed_metadata, 
+				     CODESET_RELAY);
+    ti->composed_metadata[MAX_METADATA_LEN] = 0;  // note, not LEN-1
+    num_16_bytes = (num_bytes + 15) / 16;
+    ti->composed_metadata[0] = num_16_bytes;
 }
 
 void
@@ -361,7 +439,7 @@ parse_metadata (TRACK_INFO* ti)
 {
     int eflags;
     int rc;
-    char query_string[MAX_TRACK_LEN];
+    mchar query_string[MAX_TRACK_LEN];
     Parse_Rule* rulep;
 
     /* Has any m/.../s rule matched? */
@@ -384,14 +462,18 @@ parse_metadata (TRACK_INFO* ti)
     /* Loop through rules, if we find a matching rule, then use it */
     /* For now, only default rules supported with ascii 
        regular expressions. */
-    strcpy (query_string, ti->raw_metadata);
+    debug_printf ("Converting query string to wide\n");
+    mstring_from_string (query_string, MAX_TRACK_LEN, 
+			 ti->raw_metadata, CODESET_METADATA);
     for (rulep = m_global_rule_list; rulep->cmd; rulep++) {
 	regmatch_t pmatch[MAX_SUBMATCHES+1];
 
 	eflags = 0;
 	if (rulep->cmd == PARSERULE_CMD_MATCH) {
+	    debug_mprintf (m_("Testing match rule: ") m_S m_(" vs. ") m_S m_("\n"),
+			    query_string, rulep->match);
 	    if (rulep->flags & PARSERULE_SKIP) {
-		rc = regexec(rulep->reg, query_string, 0, NULL, eflags);
+		rc = mregexec (rulep->reg, query_string, 0, NULL, eflags);
 		if (rc != 0) {
 		    /* Didn't match rule. */
 		    continue;
@@ -402,21 +484,19 @@ parse_metadata (TRACK_INFO* ti)
 		ti->have_track_info = 0;
 		return;
 	    } else if (rulep->flags & PARSERULE_SAVE) {
-		rc = regexec(rulep->reg, query_string, 0, NULL, eflags);
+		rc = mregexec (rulep->reg, query_string, 0, NULL, eflags);
 		if (rc != 0) {
 		    /* Didn't match rule. */
 		    if (!save_track_matched)
 		        ti->save_track = FALSE;
-
 		    continue;
 		}
-		if (!exclude_track_matched)
-		{
+		if (!exclude_track_matched) {
 		    ti->save_track = TRUE;
 		    save_track_matched = TRUE;
 		}
 	    } else if (rulep->flags & PARSERULE_EXCLUDE) {
-		rc = regexec(rulep->reg, query_string, 0, NULL, eflags);
+		rc = mregexec (rulep->reg, query_string, 0, NULL, eflags);
 		if (rc == 0 && !save_track_matched) {
 		    /* Rule matched => Exclude track */
 		    ti->save_track = FALSE;
@@ -424,7 +504,8 @@ parse_metadata (TRACK_INFO* ti)
 		}
 	    } else {
     		eflags = 0;
-		rc = regexec(rulep->reg, query_string, MAX_SUBMATCHES+1, pmatch, eflags);
+		rc = mregexec (rulep->reg, query_string, MAX_SUBMATCHES+1, 
+			       pmatch, eflags);
 		if (rc != 0) {
 		    /* Didn't match rule. */
 		    continue;
@@ -432,34 +513,47 @@ parse_metadata (TRACK_INFO* ti)
 		copy_rule_result (ti->artist, query_string, pmatch, rulep->artist_idx);
 		copy_rule_result (ti->title, query_string, pmatch, rulep->title_idx);
 		copy_rule_result (ti->album, query_string, pmatch, rulep->album_idx);
-		compose_metadata (ti);
-		/* GCS FIX: We don't have a track no */
 		ti->have_track_info = 1;
-		debug_printf ("Parsed track info.\nARTIST: %s\nTITLE: %s\nALBUM: %s\n",
-			ti->artist, ti->title, ti->album);
+		compose_metadata (ti);
+		/* GCS FIX: We don't have a track number. */
+		debug_mprintf (m_("Parsed track info.\n")
+			       m_("ARTIST: ") m_S m_("\n")
+			       m_("TITLE: ")  m_S m_("\n")
+			       m_("ALBUM: ")  m_S m_("\n"),
+			       ti->artist, ti->title, ti->album);
 		return;
 	    }
 	}
 	else if (rulep->cmd == PARSERULE_CMD_SUBST) {
-	    char subst_string[MAX_TRACK_LEN];
+	    mchar subst_string[MAX_TRACK_LEN];
 	    int used, left;
-	    rc = regexec(rulep->reg, query_string, 1, pmatch, eflags);
+	    debug_mprintf (m_("Testing subst rule: ") m_S m_(" vs. ") m_S m_("\n"),
+			    query_string, rulep->match);
+	    rc = mregexec (rulep->reg, query_string, 1, pmatch, eflags);
 	    if (rc != 0) {
 		/* Didn't match rule. */
 		continue;
 	    }
 	    /* Update the query string and continue. */
-	    strncpy(subst_string, query_string, pmatch[0].rm_so);
+	    debug_printf ("Matched at (%d,%d)\n", 
+			    pmatch[0].rm_so, pmatch[0].rm_eo);
+	    mstrncpy (subst_string, query_string, pmatch[0].rm_so + 1);
+	    debug_mprintf (m_("(1) subst_string = ") m_S m_("\n"), subst_string);
 	    used = pmatch[0].rm_so;
 	    left = MAX_TRACK_LEN - used;
-	    strncpy(subst_string + used, rulep->subst, left-1);
-	    used += strlen (rulep->subst);
+	    mstrncpy (subst_string + used, rulep->subst, left);
+	    debug_mprintf (m_("(2) subst_string = ") m_S m_("\n"), subst_string);
+	    used += mstrlen (rulep->subst);
 	    left = MAX_TRACK_LEN - used;
-	    sr_strncpy(subst_string + used, query_string + pmatch[0].rm_eo, left);
-	    sr_strncpy(query_string, subst_string, MAX_TRACK_LEN);
+	    mstrncpy (subst_string + used, 
+		      query_string + pmatch[0].rm_eo, left);
+	    debug_mprintf (m_("(3) subst_string = ") m_S m_("\n"), subst_string);
+	    mstrncpy (query_string, subst_string, MAX_TRACK_LEN);
+	    debug_mprintf (m_("(4) query_string = ") m_S m_("\n"), query_string);
 	}
     }
     debug_printf ("Fell through while parsing data...\n");
-    strcpy (ti->title, ti->raw_metadata);
+    mstrncpy (ti->title, query_string, MAX_TRACK_LEN);
     ti->have_track_info = 1;
+    compose_metadata (ti);
 }
