@@ -41,7 +41,7 @@
 #include "filelib.h"
 #include "socklib.h"
 #include "http.h"
-#include "util.h"
+#include "mchar.h"
 #include "findsep.h"
 #include "relaylib.h"
 #include "rip_manager.h"
@@ -57,10 +57,7 @@
 static void			ripthread(void *bla);
 static error_code		start_relay(int content_type);
 static void			post_status(int status);
-//static error_code		start_track(char *track);
-//static error_code		rip_manager_end_track(char *track);
-//static error_code		rip_manager_put_data(char *buf, int size);
-static error_code		start_ripping();
+static error_code start_ripping (RIP_MANAGER_OPTIONS* rmo);
 static void			destroy_subsystems();
 
 
@@ -73,7 +70,7 @@ static HSOCKET			m_sock;
 static void			(*m_destroy_func)();
 #endif
 static RIP_MANAGER_INFO		m_ripinfo;		// used for UPDATE callback messages
-static RIP_MANAGER_OPTIONS	m_options;		// local copy of the options passed to rip_manager_start()
+//static RIP_MANAGER_OPTIONS	m_options;		// local copy of the options passed to rip_manager_start()
 static THREAD_HANDLE		m_hthread;		// rip thread handle
 static void			(*m_status_callback)(int message, void *data);
 static BOOL			m_ripping = FALSE;
@@ -174,11 +171,13 @@ rip_manager_get_error_str(error_code code)
     return m_error_str[-code];
 }
 
+#if defined (commentout)
 u_short
 rip_mananger_get_relay_port()
 {
-    return m_options.relay_port;
+    return rmo->relay_port;
 }
+#endif
 
 void
 post_error(int err)
@@ -198,6 +197,15 @@ post_status(int status)
     m_status_callback(RM_UPDATE, &m_ripinfo);
 }
 
+void
+compose_console_string (TRACK_INFO* ti)
+{
+    mchar console_string[SR_MAX_PATH];
+    msnprintf (console_string, SR_MAX_PATH, m_S m_(" - ") m_S, 
+	       ti->artist, ti->title);
+    string_from_mstring (m_ripinfo.filename, SR_MAX_PATH, console_string,
+			 CODESET_LOCALE);
+}
 
 /* 
  * This is called by ripstream when we get a new track. 
@@ -229,8 +237,7 @@ rip_manager_start_track (TRACK_INFO* ti, int track_count)
     m_ripinfo.track_count = track_count;
 
     /* Compose the string for the console output */
-    snprintf (m_ripinfo.filename, SR_MAX_PATH, "%s - %s", 
-	      ti->artist, ti->title);
+    compose_console_string (ti);
     m_ripinfo.filename[SR_MAX_PATH-1] = '\0';
     m_status_callback(RM_NEW_TRACK, (void*) m_ripinfo.filename);
     post_status(0);
@@ -244,18 +251,20 @@ rip_manager_start_track (TRACK_INFO* ti, int track_count)
  * the first track is *never* complete.
  */
 error_code
-rip_manager_end_track(TRACK_INFO* ti)
+rip_manager_end_track (RIP_MANAGER_OPTIONS* rmo, TRACK_INFO* ti)
 {
+    mchar mfullpath[SR_MAX_PATH];
     char fullpath[SR_MAX_PATH];
 
     if (m_write_data) {
-        filelib_end (ti, m_options.overwrite, 
-		     GET_TRUNCATE_DUPS(m_options.flags),
-		     fullpath);
+        filelib_end (ti, rmo->overwrite, 
+		     GET_TRUNCATE_DUPS(rmo->flags),
+		     mfullpath);
     }
-
     post_status(0);
-    m_status_callback(RM_TRACK_DONE, (void*)fullpath);
+
+    string_from_mstring (fullpath, SR_MAX_PATH, mfullpath, CODESET_FILESYS);
+    m_status_callback (RM_TRACK_DONE, (void*)fullpath);
 
     return SR_SUCCESS;
 }
@@ -327,21 +336,25 @@ client_relay_header_release (char *ch)
 }
 
 void
-ripthread (void *notused)
+ripthread (void *input_arg)
 {
     error_code ret;
+    RIP_MANAGER_OPTIONS* rmo = (RIP_MANAGER_OPTIONS*) input_arg;
+    debug_printf ("ripthread RMO_URL = %s\n", rmo->url);
 
-    if ((ret = start_ripping()) != SR_SUCCESS) {
-	threadlib_signal_sem(&m_started_sem);
+    if ((ret = start_ripping(rmo)) != SR_SUCCESS) {
+	debug_printf ("Ripthread did start_ripping()\n");
+	threadlib_signal_sem (&m_started_sem);
 	post_error(ret);
 	goto DONE;
     }
     m_status_callback(RM_STARTED, (void *)NULL);
     post_status(RM_STATUS_BUFFERING);
+    debug_printf ("Ripthread did initialization\n");
     threadlib_signal_sem(&m_started_sem);
 
     while (TRUE) {
-        ret = ripstream_rip();
+        ret = ripstream_rip(rmo);
 
 	/* If the user told us to stop, well, then we bail */
 	if (!m_ripping)
@@ -357,11 +370,11 @@ ripthread (void *notused)
 	 * wnd locks, etc.. all i know is that this appeared to work
 	 */
         /* GCS Aug 23, 2003: m_bytes_ripped can still overflow */
-	if (m_bytes_ripped/1000000 >= (m_options.maxMB_rip_size) &&
-	    GET_CHECK_MAX_BYTES(m_options.flags)) {
-	    socklib_close(&m_sock);
-	    destroy_subsystems();
-	    post_error(SR_ERROR_MAX_BYTES_RIPPED);
+	if (m_bytes_ripped/1000000 >= (rmo->maxMB_rip_size) &&
+		GET_CHECK_MAX_BYTES (rmo->flags)) {
+	    socklib_close (&m_sock);
+	    destroy_subsystems ();
+	    post_error (SR_ERROR_MAX_BYTES_RIPPED);
 	    break;
 	}
 
@@ -380,13 +393,10 @@ ripthread (void *notused)
 		  ret == SR_ERROR_TIMEOUT || 
 		  ret == SR_ERROR_NO_TRACK_INFO || 
 		  ret == SR_ERROR_SELECT_FAILED) && 
-		 GET_AUTO_RECONNECT(m_options.flags)) {
-	    /*
-	     * Try to reconnect, if thats what the user wants
-	     */
+		 GET_AUTO_RECONNECT (rmo->flags)) {
+	    /* Try to reconnect, if thats what the user wants */
 	    post_status(RM_STATUS_RECONNECTING);
-	    while(m_ripping) {
-
+	    while (m_ripping) {
 		// Hopefully this solves a lingering bug 
 		// with auto-reconnects failing to bind to the relay port
 		// (a long with other unknown problems)
@@ -404,7 +414,7 @@ ripthread (void *notused)
 		relaylib_shutdown();
 		filelib_shutdown();
 		ripstream_destroy();
-		ret = start_ripping();
+		ret = start_ripping (rmo);
 		if (ret == SR_SUCCESS)
 		    break;
 
@@ -431,6 +441,7 @@ ripthread (void *notused)
  DONE:
     m_status_callback(RM_DONE, &m_ripinfo);
     m_ripping = FALSE;
+    debug_printf ("ripthread() exiting!\n");
 }
 
 
@@ -438,8 +449,10 @@ void
 rip_manager_stop()
 {
     // Make sure this function isn't getting called twice
-    if (!m_ripping)
-	    return;
+    if (!m_ripping) {
+	debug_printf ("rip_manager_stop() called while not m_ripping\n");
+	return;
+    }
     
     // Make sure the ripping started before we try to stop
     debug_printf ("Waiting for m_started_sem...\n");
@@ -480,49 +493,53 @@ destroy_subsystems()
     filelib_shutdown();
 }
 
-int
-create_pls_file()
+static int
+create_pls_file (RIP_MANAGER_OPTIONS* rmo)
 {
     FILE *fid;
 
-    if  ('\0' == m_options.relay_ip[0]) {
+    if  ('\0' == rmo->relay_ip[0]) {
 	fprintf(stderr, "can not determine relaying ip, pass ip to -r \n");
 	return -1;
     }
 
-    fid = fopen(m_options.pls_file, "w");
+    fid = fopen (rmo->pls_file, "w");
 
     if (NULL == fid) {
 	fprintf(stderr, "could not create playlist file '%s': %d '%s'\n",
-		m_options.pls_file, errno, strerror(errno));
+		rmo->pls_file, errno, strerror(errno));
     } else {
 	fprintf(fid, "[playlist]\n");
 	fprintf(fid, "NumberOfEntries=1\n");
-	fprintf(fid, "File1=http://%s:%d\n", m_options.relay_ip, m_options.relay_port);
+	fprintf(fid, "File1=http://%s:%d\n", rmo->relay_ip, rmo->relay_port);
 	fclose(fid);
     }
-		
+
     return 0;
 }
 
-error_code
-start_ripping()
+static error_code
+start_ripping (RIP_MANAGER_OPTIONS* rmo)
 {
     error_code ret;
-    char *pproxy = m_options.proxyurl[0] ? m_options.proxyurl : NULL;
+
+    char *pproxy = rmo->proxyurl[0] ? rmo->proxyurl : NULL;
+    debug_printf ("start_ripping: checkpoint 1\n");
 
     /* If proxy URL not spec'd on command line (or plugin field), 
        check the environment variable */
     if (!pproxy) {
 	char const *env_http_proxy = getenv ("http_proxy");
 	if (env_http_proxy) {
-	    strncpy(m_options.proxyurl, env_http_proxy, MAX_URL_LEN);
+	    strncpy (rmo->proxyurl, env_http_proxy, MAX_URL_LEN);
 	}
     }
 
+    debug_printf ("start_ripping: checkpoint 2\n");
+
     /* Connect to the stream */
-    ret = httplib_sc_connect(&m_sock, m_options.url, pproxy, &m_http_info, 
-			  m_options.useragent, m_options.if_name);
+    ret = httplib_sc_connect (&m_sock, rmo->url, pproxy, &m_http_info, 
+			      rmo->useragent, rmo->if_name);
     if (ret != SR_SUCCESS) {
 	goto RETURN_ERR;
     }
@@ -544,17 +561,17 @@ start_ripping()
 
     /* Initialize file writing code. */
     ret = filelib_init
-	    (GET_INDIVIDUAL_TRACKS(m_options.flags),
-	     GET_COUNT_FILES(m_options.flags),
-	     m_options.count_start,
-	     GET_KEEP_INCOMPLETE(m_options.flags),
-	     GET_SINGLE_FILE_OUTPUT(m_options.flags),
+	    (GET_INDIVIDUAL_TRACKS (rmo->flags),
+	     GET_COUNT_FILES(rmo->flags),
+	     rmo->count_start,
+	     GET_KEEP_INCOMPLETE(rmo->flags),
+	     GET_SINGLE_FILE_OUTPUT(rmo->flags),
 	     m_http_info.content_type, 
-	     m_options.output_directory,
-	     m_options.output_pattern,
-	     m_options.showfile_pattern,
-	     GET_SEPERATE_DIRS(m_options.flags),
-	     GET_DATE_STAMP(m_options.flags),
+	     rmo->output_directory,
+	     rmo->output_pattern,
+	     rmo->showfile_pattern,
+	     GET_SEPERATE_DIRS(rmo->flags),
+	     GET_DATE_STAMP(rmo->flags),
 	     m_http_info.icy_name);
     if (ret != SR_SUCCESS)
 	goto RETURN_ERR;
@@ -566,9 +583,9 @@ start_ripping()
 
     /* Start up external program to get metadata. */
     m_ripinfo.ep = 0;
-    if (GET_EXTERNAL_CMD(m_options.flags)) {
-	debug_printf ("Spawn external: %s\n", m_options.ext_cmd);
-	m_ripinfo.ep = spawn_external (m_options.ext_cmd);
+    if (GET_EXTERNAL_CMD(rmo->flags)) {
+	debug_printf ("Spawn external: %s\n", rmo->ext_cmd);
+	m_ripinfo.ep = spawn_external (rmo->ext_cmd);
 	if (m_ripinfo.ep) {
 	    debug_printf ("Spawn external succeeded\n");
 	} else {
@@ -581,15 +598,16 @@ start_ripping()
      */
     ripstream_destroy();
     ret = ripstream_init(m_sock, 
-			 GET_MAKE_RELAY(m_options.flags),
-			 m_options.timeout, 
+			 GET_MAKE_RELAY(rmo->flags),
+			 rmo->timeout, 
 			 m_http_info.icy_name,
-			 m_options.dropcount,
-			 &m_options.sp_opt,
+			 rmo->dropcount,
+			 &rmo->sp_opt,
 			 m_ripinfo.bitrate, 
 			 m_http_info.meta_interval,
 			 m_http_info.content_type, 
-			 GET_ADD_ID3(m_options.flags),
+			 GET_ADD_ID3V1(rmo->flags),
+			 GET_ADD_ID3V2(rmo->flags),
 			 m_ripinfo.ep);
     if (ret != SR_SUCCESS) {
 	ripstream_destroy();
@@ -603,22 +621,22 @@ start_ripping()
      * the stream we are relaying.. this just sets the header to 
      * something very simulare to what we got from the stream.
      */
-    if (GET_MAKE_RELAY (m_options.flags)) {
-	int new_port = 0;
-	ret = relaylib_init(GET_SEARCH_PORTS(m_options.flags), 
-			    m_options.relay_port, m_options.max_port, 
-			    &new_port, m_options.if_name, 
-			    m_options.max_connections, m_options.relay_ip,
+    if (GET_MAKE_RELAY (rmo->flags)) {
+	u_short new_port = 0;
+	ret = relaylib_init(GET_SEARCH_PORTS(rmo->flags), 
+			    rmo->relay_port, rmo->max_port, 
+			    &new_port, rmo->if_name, 
+			    rmo->max_connections, rmo->relay_ip,
 			    m_http_info.meta_interval != NO_META_INTERVAL);
 	if (ret != SR_SUCCESS) {
 		goto RETURN_ERR;
 	}
 
-	m_options.relay_port = new_port;
+	rmo->relay_port = new_port;
 	start_relay(m_http_info.content_type);
 
-	if (0 != m_options.pls_file[0]) {
-		create_pls_file(new_port);
+	if (0 != rmo->pls_file[0]) {
+		create_pls_file (rmo);
 	}
     }
     post_status(RM_STATUS_BUFFERING);
@@ -634,8 +652,8 @@ RETURN_ERR:
 }
 
 error_code
-rip_manager_start(void (*status_callback)(int message, void *data), 
-			     RIP_MANAGER_OPTIONS *options)
+rip_manager_start (void (*status_callback)(int message, void *data), 
+			     RIP_MANAGER_OPTIONS *rmo)
 {
     int ret = 0;
     m_started_sem = threadlib_create_sem();
@@ -644,32 +662,29 @@ rip_manager_start(void (*status_callback)(int message, void *data),
 
     m_ripping = TRUE;
 
-    initialize_default_locale(&options->cs_opt);
+    register_codesets (&rmo->cs_opt);
 
-    if (!options)
+    if (!rmo)
 	return SR_ERROR_INVALID_PARAM;
 
     socklib_init();
 
     init_error_strings();
     m_status_callback = status_callback;
-#if defined (commentout)
-    m_in.get_input_data = myrecv;
-    m_destroy_func = NULL;
-#endif
     m_bytes_ripped = 0;
 
     /* Initialize the parsing rules */
-    init_metadata_parser (options->rules_file);
+    init_metadata_parser (rmo->rules_file);
 
+#if defined (commentout)
     /* Get a local copy of the options passed */
     memcpy(&m_options, options, sizeof(RIP_MANAGER_OPTIONS));
+#endif
 
     /* Start the ripping thread */
     m_ripping = TRUE;
-    if ((ret = threadlib_beginthread(&m_hthread, ripthread)) != SR_SUCCESS)
-	return ret;
-    return SR_SUCCESS;
+    ret = threadlib_beginthread (&m_hthread, ripthread, (void*) rmo);
+    return ret;
 }
 
 /* Winamp plugin needs to get content type */
@@ -698,48 +713,53 @@ overwrite_opt_to_string (enum OverwriteOpt oo)
 }
 
 void
-set_rip_manager_options_defaults (RIP_MANAGER_OPTIONS *m_opt)
+set_rip_manager_options_defaults (RIP_MANAGER_OPTIONS *rmo)
 {
-    m_opt->relay_port = 8000;
-    m_opt->max_port = 18000;
-    m_opt->flags = OPT_AUTO_RECONNECT | 
+    debug_printf ("- set_rip_manager_options_defaults -\n");
+    rmo->url[0] = 0;
+    rmo->proxyurl[0] = 0;
+    strcpy(rmo->output_directory, "./");
+    rmo->output_pattern[0] = 0;
+    rmo->showfile_pattern[0] = 0;
+    rmo->if_name[0] = 0;
+    rmo->rules_file[0] = 0;
+    rmo->pls_file[0] = 0;
+    rmo->relay_ip[0] = 0;
+    rmo->relay_port = 8000;
+    rmo->max_port = 18000;
+    rmo->max_connections = 1;
+    rmo->maxMB_rip_size = 0;
+    rmo->flags = OPT_AUTO_RECONNECT | 
 	    OPT_SEPERATE_DIRS | 
 	    OPT_SEARCH_PORTS |
-	    OPT_ADD_ID3 |
+	    /* OPT_ADD_ID3V1 | -- removed starting 1.62-beta-2 */
+	    OPT_ADD_ID3V2 |
 	    OPT_INDIVIDUAL_TRACKS;
+    strcpy(rmo->useragent, "sr-POSIX/" SRVERSION);
+
+    // Defaults for splitpoint - times are in ms
+    rmo->sp_opt.xs = 1;
+    rmo->sp_opt.xs_min_volume = 1;
+    rmo->sp_opt.xs_silence_length = 1000;
+    rmo->sp_opt.xs_search_window_1 = 6000;
+    rmo->sp_opt.xs_search_window_2 = 6000;
+    rmo->sp_opt.xs_offset = 0;
+    rmo->sp_opt.xs_padding_1 = 300;
+    rmo->sp_opt.xs_padding_2 = 300;
 
     /* GCS FIX: What is the difference between this timeout 
        and the one used in setsockopt()? */
 #if defined (commentout)
-    m_opt->timeout = 0;
+    rmo->timeout = 0;
 #endif
-    m_opt->timeout = 15;
-    m_opt->max_connections = 1;
-
-    strcpy(m_opt->output_directory, "./");
-    m_opt->output_pattern[0] = 0;
-    m_opt->relay_ip[0] = 0;
-    m_opt->pls_file[0] = 0;
-    m_opt->proxyurl[0] = 0;
-    m_opt->url[0] = 0;
-    m_opt->showfile_pattern[0] = 0;
-    m_opt->rules_file[0] = 0;
-    strcpy(m_opt->useragent, "sr-POSIX/" SRVERSION);
-    m_opt->overwrite = OVERWRITE_LARGER;
-    m_opt->dropcount = 0;
-    m_opt->ext_cmd[0] = 0;
-
-    // Defaults for splitpoint
-    // Times are in ms
-    m_opt->sp_opt.xs = 1;
-    m_opt->sp_opt.xs_min_volume = 1;
-    m_opt->sp_opt.xs_silence_length = 1000;
-    m_opt->sp_opt.xs_search_window_1 = 6000;
-    m_opt->sp_opt.xs_search_window_2 = 6000;
-    m_opt->sp_opt.xs_offset = 0;
-    m_opt->sp_opt.xs_padding_1 = 300;
-    m_opt->sp_opt.xs_padding_2 = 300;
+    rmo->timeout = 15;
+    rmo->dropcount = 0;
 
     // Defaults for codeset
-    memset (&m_opt->cs_opt, 0, sizeof(CODESET_OPTIONS));
+    memset (&rmo->cs_opt, 0, sizeof(CODESET_OPTIONS));
+    set_codesets_default (&rmo->cs_opt);
+
+    rmo->count_start = 0;
+    rmo->overwrite = OVERWRITE_LARGER;
+    rmo->ext_cmd[0] = 0;
 }
