@@ -20,7 +20,7 @@
 #include "external.h"
 #include "srtypes.h"
 
-#define SRVERSION	"1.61.27"
+#define SRVERSION	"1.62.1"
 
 #if defined (WIN32)
 #define SRPLATFORM      "windows"
@@ -61,7 +61,7 @@ typedef struct RIP_MANAGER_INFOst
     char filename[SR_MAX_PATH];  // it's not the filename, it's the trackname
     u_long filesize;
     int	status;
-    int  track_count;
+    int track_count;
     External_Process* ep;
 } RIP_MANAGER_INFO;
 
@@ -72,7 +72,7 @@ typedef struct RIP_MANAGER_INFOst
 #define OPT_SEARCH_PORTS	0x00000008		// relay server should search for a open port
 #define OPT_MAKE_RELAY		0x00000010		// don't make a relay server
 #define OPT_COUNT_FILES		0x00000020		// add a index counter to the filenames
-#define OPT_ADD_ID3		0x00000040		// add ID3 info to the mp3s (thanks oddsock)
+#define OPT_OBSOLETE		0x00000040		// Used to be OPT_ADD_ID3, now ignored
 #define OPT_DATE_STAMP		0x00000100		// add a date stamp to the output directory
 #define OPT_CHECK_MAX_BYTES	0x00000200		// use the maxMB_rip_size value to know how much to rip
 #define OPT_KEEP_INCOMPLETE	0x00000400		// overwrite files in the incomplete directory, add counter instead
@@ -80,16 +80,19 @@ typedef struct RIP_MANAGER_INFOst
 #define OPT_TRUNCATE_DUPS	0x00001000		// truncate file in the incomplete directory already present in complete
 #define OPT_INDIVIDUAL_TRACKS	0x00002000		// should we write the individual tracks?
 #define OPT_EXTERNAL_CMD	0x00004000		// use external command to get metadata?
+#define OPT_ADD_ID3V1		0x00008000		// Add ID3V1
+#define OPT_ADD_ID3V2		0x00010000		// Add ID3V2
 
 #define OPT_FLAG_ISSET(flags, opt)	    ((flags & opt) > 0)
-#define OPT_FLAG_SET(flags, opt)	    (flags =| opt)
+// #define OPT_FLAG_SET(flags, opt)	    (flags =| opt)
+#define OPT_FLAG_SET(flags, opt, val)	    (val ? (flags |= opt) : (flags &= (~opt)))
 
 #define GET_AUTO_RECONNECT(flags)		(OPT_FLAG_ISSET(flags, OPT_AUTO_RECONNECT))
 #define GET_SEPERATE_DIRS(flags)		(OPT_FLAG_ISSET(flags, OPT_SEPERATE_DIRS))
 #define GET_SEARCH_PORTS(flags)			(OPT_FLAG_ISSET(flags, OPT_SEARCH_PORTS))
 #define GET_MAKE_RELAY(flags)			(OPT_FLAG_ISSET(flags, OPT_MAKE_RELAY))
 #define GET_COUNT_FILES(flags)			(OPT_FLAG_ISSET(flags, OPT_COUNT_FILES))
-#define GET_ADD_ID3(flags)			(OPT_FLAG_ISSET(flags, OPT_ADD_ID3))
+// #define GET_ADD_ID3(flags)			(OPT_FLAG_ISSET(flags, OPT_ADD_ID3))
 #define GET_DATE_STAMP(flags)			(OPT_FLAG_ISSET(flags, OPT_DATE_STAMP))
 #define GET_CHECK_MAX_BYTES(flags)		(OPT_FLAG_ISSET(flags, OPT_CHECK_MAX_BYTES))
 #define GET_KEEP_INCOMPLETE(flags)		(OPT_FLAG_ISSET(flags, OPT_KEEP_INCOMPLETE))
@@ -97,14 +100,17 @@ typedef struct RIP_MANAGER_INFOst
 #define GET_TRUNCATE_DUPS(flags)		(OPT_FLAG_ISSET(flags, OPT_TRUNCATE_DUPS))
 #define GET_INDIVIDUAL_TRACKS(flags)		(OPT_FLAG_ISSET(flags, OPT_INDIVIDUAL_TRACKS))
 #define GET_EXTERNAL_CMD(flags)			(OPT_FLAG_ISSET(flags, OPT_EXTERNAL_CMD))
+#define GET_ADD_ID3V1(flags)			(OPT_FLAG_ISSET(flags, OPT_ADD_ID3V1))
+#define GET_ADD_ID3V2(flags)			(OPT_FLAG_ISSET(flags, OPT_ADD_ID3V2))
 
+#if defined (commentout)
 #define SET_AUTO_RECONNECT(flags)		(OPT_FLAG_SET(flags, OPT_AUTO_RECONNECT))
 #define SET_SEPERATE_DIRS(flags)		(OPT_FLAG_SET(flags, OPT_SEPERATE_DIRS))
 #define SET_OVER_WRITE_TRACKS(flags)		(OPT_FLAG_SET(flags, OPT_OVER_WRITE_TRACKS))
 #define SET_SEARCH_PORTS(flags)			(OPT_FLAG_SET(flags, OPT_SEARCH_PORTS))
 #define SET_MAKE_RELAY(flags)			(OPT_FLAG_SET(flags, OPT_MAKE_RELAY))
 #define SET_COUNT_FILES(flags)			(OPT_FLAG_SET(flags, OPT_COUNT_FILES))
-#define SET_ADD_ID3(flags)			(OPT_FLAG_SET(flags, OPT_ADD_ID3))
+// #define SET_ADD_ID3(flags)			(OPT_FLAG_SET(flags, OPT_ADD_ID3))
 #define SET_DATE_STAMP(flags)			(OPT_FLAG_SET(flags, OPT_DATE_STAMP))
 #define SET_CHECK_MAX_BYTES(flags)		(OPT_FLAG_SET(flags, OPT_CHECK_MAX_BYTES))
 #define SET_KEEP_INCOMPLETE(flags)		(OPT_FLAG_SET(flags, OPT_KEEP_INCOMPLETE))
@@ -112,6 +118,9 @@ typedef struct RIP_MANAGER_INFOst
 #define SET_TRUNCATE_DUPS(flags)		(OPT_FLAG_SET(flags, OPT_TRUNCATE_DUPS))
 #define SET_INDIVIDUAL_TRACKS(flags)		(OPT_FLAG_SET(flags, OPT_INDIVIDUAL_TRACKS))
 #define SET_EXTERNAL_CMD(flags)			(OPT_FLAG_SET(flags, OPT_EXTERNAL_CMD))
+#define SET_ADD_ID3V1(flags)			(OPT_FLAG_SET(flags, OPT_ADD_ID3V1))
+#define SET_ADD_ID3V2(flags)			(OPT_FLAG_SET(flags, OPT_ADD_ID3V2))
+#endif
 
 typedef struct RIP_MANAGER_OPTIONSst
 {
@@ -130,14 +139,15 @@ typedef struct RIP_MANAGER_OPTIONSst
                                         //  rely .pls file
     char relay_ip[SR_MAX_PATH];		// optional, ip to bind relaying 
                                         //  socket to
-    int relay_port;			// port to use for the relay server
+    u_short relay_port;			// port to use for the relay server
+					//  GCS 3/30/07 change to u_short
     u_short max_port;			// highest port the relay server 
                                         //  can look if it needs to search
     int max_connections;                // max number of connections 
                                         //  to relay stream
     u_long maxMB_rip_size;		// max number of megabytes that 
                                         //  can by writen out before we stop
-    u_short flags;			// all booleans logically OR'd 
+    u_long flags;			// all booleans logically OR'd 
                                         //  together (see above)
     char useragent[MAX_USERAGENT_STR];	// optional, use a different useragent
     SPLITPOINT_OPTIONS sp_opt;		// options for splitpoint rules
@@ -162,13 +172,13 @@ typedef struct ERROR_INFOst
 
 /* Public functions */
 char *rip_manager_get_error_str(int code);
-u_short rip_mananger_get_relay_port();	
+//u_short rip_mananger_get_relay_port();	
 void set_rip_manager_options_defaults (RIP_MANAGER_OPTIONS *m_opt);
 error_code rip_manager_start (void (*status_callback)(int message, void *data), 
 			     RIP_MANAGER_OPTIONS *options);
 void rip_manager_stop();
 error_code rip_manager_start_track (TRACK_INFO* ti, int track_count);
-error_code rip_manager_end_track(TRACK_INFO* ti);
+error_code rip_manager_end_track(RIP_MANAGER_OPTIONS* rmo, TRACK_INFO* ti);
 error_code rip_manager_put_data(char *buf, int size);
 error_code rip_manager_put_raw_data(char *buf, int size);
 
