@@ -1,6 +1,7 @@
 #ifndef __RIPLIB_H__
 #define __RIPLIB_H__
 
+#include "rip_manager.h"
 #include "srtypes.h"
 #include "socklib.h"
 
@@ -14,9 +15,10 @@ ripstream_init (HSOCKET sock,
 		int bitrate, 
 		int meta_interval, 
 		int content_type, 
-		BOOL addID3tag,
+		BOOL add_id3v1,
+		BOOL add_id3v2,
 		External_Process* ep);
-error_code ripstream_rip();
+error_code ripstream_rip(RIP_MANAGER_OPTIONS* rmo);
 void ripstream_destroy();
 
 
